import { ParsedNode } from '../nodes/Node';
import { Pair } from '../nodes/Pair';
import { ComposeContext } from './compose-node';
export declare function mapIncludes(ctx: ComposeContext, items: Pair<ParsedNode>[], search: ParsedNode): boolean;
