/**
 * Stringifies a comment.
 *
 * Empty comment lines are left empty,
 * lines consisting of a single space are replaced by `#`,
 * and all other lines are prefixed with a `#`.
 */
export declare const stringifyComment: (str: string) => string;
export declare function indentComment(comment: string, indent: string): string;
export declare const lineComment: (str: string, indent: string, comment: string) => string;
